/**
 * In order to provide fast and flexible logging, this project uses Cocoa Lumberjack.
 *
 * The Google Code page has a wealth of documentation if you have any questions.
 * https://github.com/robbiehanson/CocoaLumberjack
 *
 * Here's what you need to know concerning how logging is setup for IntAirAct:
 *
 * There are 4 log levels:
 * - Error
 * - Warning
 * - Info
 * - Verbose
 *
 * In addition to this, there is a Trace flag that can be enabled.
 * When tracing is enabled, it spits out the methods that are being called.
 *
 * Please note that tracing is separate from the log levels.
 * For example, one could set the log level to warning, and enable tracing.
 *
 * All logging is asynchronous, except errors.
 * To use logging within your own custom files, follow the steps below.
 *
 * Step 1:
 * Import this header in your implementation file:
 *
 * #import "SDLogging.h"
 *
 * Step 2:
 * Define your logging level in your implementation file:
 *
 * // Log levels: off, error, warn, info, verbose
 * static const int sdLogLevel = SD_LOG_LEVEL_VERBOSE;
 *
 * If you wish to enable tracing, you could do something like this:
 *
 * // Debug levels: off, error, warn, info, verbose
 * static const int sdLogLevel = SD_LOG_LEVEL_INFO | SD_LOG_FLAG_TRACE;
 *
 * Step 3:
 * Replace your NSLog statements with SDLog statements according to the severity of the message.
 *
 * NSLog(@"Fatal error, no dohickey found!"); -> SDLogError(@"Fatal error, no dohickey found!");
 *
 * SDLog works exactly the same as NSLog.
 * This means you can pass it multiple variables just like NSLog.
 **/

#import <CocoaLumberjack/DDLog.h>

// Define logging context for every log message coming from the SD server.
// The logging context can be extracted from the DDLogMessage from within the logging framework,
// which gives loggers, formatters, and filters the ability to optionally process them differently.

#define SD_LOG_CONTEXT 80

// Configure log levels.

#define SD_LOG_FLAG_ERROR   (1 << 0) // 0...00001
#define SD_LOG_FLAG_WARN    (1 << 1) // 0...00010
#define SD_LOG_FLAG_INFO    (1 << 2) // 0...00100
#define SD_LOG_FLAG_VERBOSE (1 << 3) // 0...01000

#define SD_LOG_LEVEL_OFF     0                                              // 0...00000
#define SD_LOG_LEVEL_ERROR   (SD_LOG_LEVEL_OFF   | SD_LOG_FLAG_ERROR)   // 0...00001
#define SD_LOG_LEVEL_WARN    (SD_LOG_LEVEL_ERROR | SD_LOG_FLAG_WARN)    // 0...00011
#define SD_LOG_LEVEL_INFO    (SD_LOG_LEVEL_WARN  | SD_LOG_FLAG_INFO)    // 0...00111
#define SD_LOG_LEVEL_VERBOSE (SD_LOG_LEVEL_INFO  | SD_LOG_FLAG_VERBOSE) // 0...01111
// Setup fine grained logging.
// The first 4 bits are being used by the standard log levels (0 - 3)
//
// We're going to add tracing, but NOT as a log level.
// Tracing can be turned on and off independently of log level.

#define SD_LOG_FLAG_TRACE   (1 << 4) // 0...10000

// Setup the usual boolean macros.

#define SD_LOG_ERROR   (sdLogLevel & SD_LOG_FLAG_ERROR)
#define SD_LOG_WARN    (sdLogLevel & SD_LOG_FLAG_WARN)
#define SD_LOG_INFO    (sdLogLevel & SD_LOG_FLAG_INFO)
#define SD_LOG_VERBOSE (sdLogLevel & SD_LOG_FLAG_VERBOSE)
#define SD_LOG_TRACE   (sdLogLevel & SD_LOG_FLAG_TRACE)

// Configure asynchronous logging.
// We follow the default configuration,
// but we reserve a special macro to easily disable asynchronous logging for debugging purposes.

#define SD_LOG_ASYNC_ENABLED   YES

#define SD_LOG_ASYNC_ERROR   ( NO && SD_LOG_ASYNC_ENABLED)
#define SD_LOG_ASYNC_WARN    (YES && SD_LOG_ASYNC_ENABLED)
#define SD_LOG_ASYNC_INFO    (YES && SD_LOG_ASYNC_ENABLED)
#define SD_LOG_ASYNC_VERBOSE (YES && SD_LOG_ASYNC_ENABLED)
#define SD_LOG_ASYNC_TRACE   (YES && SD_LOG_ASYNC_ENABLED)

// Define logging primitives.

#define SDLogError(frmt, ...)    LOG_OBJC_MAYBE(SD_LOG_ASYNC_ERROR,   sdLogLevel, SD_LOG_FLAG_ERROR,  \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogError2(mess)         LOG_OBJC_MAYBE(SD_LOG_ASYNC_INFO,    sdLogLevel, SD_LOG_FLAG_INFO,    \
SD_LOG_CONTEXT, @"%@[%p]: %@", THIS_FILE, self, mess)

#define SDLogWarn(frmt, ...)     LOG_OBJC_MAYBE(SD_LOG_ASYNC_WARN,    sdLogLevel, SD_LOG_FLAG_WARN,   \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogInfo(frmt, ...)     LOG_OBJC_MAYBE(SD_LOG_ASYNC_INFO,    sdLogLevel, SD_LOG_FLAG_INFO,    \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogInfo3(mess)         LOG_OBJC_MAYBE(SD_LOG_ASYNC_INFO,    sdLogLevel, SD_LOG_FLAG_INFO,    \
SD_LOG_CONTEXT, @"%@[%p]: %@", THIS_FILE, self, mess)

#define SDLogVerbose(frmt, ...)  LOG_OBJC_MAYBE(SD_LOG_ASYNC_VERBOSE, sdLogLevel, SD_LOG_FLAG_VERBOSE, \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogVerbose3(mess)      LOG_OBJC_MAYBE(SD_LOG_ASYNC_INFO,    sdLogLevel, SD_LOG_FLAG_VERBOSE, \
SD_LOG_CONTEXT, @"%@[%p]: %@", THIS_FILE, self, mess)

#define SDLogTrace()             LOG_OBJC_MAYBE(SD_LOG_ASYNC_TRACE,   sdLogLevel, SD_LOG_FLAG_TRACE, \
SD_LOG_CONTEXT, @"%@[%p]: %@", THIS_FILE, self, THIS_METHOD)

#define SDLogTrace2(frmt, ...)   LOG_OBJC_MAYBE(SD_LOG_ASYNC_TRACE,   sdLogLevel, SD_LOG_FLAG_TRACE, \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogTrace3(mess)        LOG_OBJC_MAYBE(SD_LOG_ASYNC_TRACE,   sdLogLevel, SD_LOG_FLAG_TRACE, \
SD_LOG_CONTEXT, @"%@[%p]: %@", THIS_FILE, self, mess)


#define SDLogCError(frmt, ...)      LOG_C_MAYBE(SD_LOG_ASYNC_ERROR,   sdLogLevel, SD_LOG_FLAG_ERROR,   \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogCWarn(frmt, ...)       LOG_C_MAYBE(SD_LOG_ASYNC_WARN,    sdLogLevel, SD_LOG_FLAG_WARN,    \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogCInfo(frmt, ...)       LOG_C_MAYBE(SD_LOG_ASYNC_INFO,    sdLogLevel, SD_LOG_FLAG_INFO,    \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogCVerbose(frmt, ...)    LOG_C_MAYBE(SD_LOG_ASYNC_VERBOSE, sdLogLevel, SD_LOG_FLAG_VERBOSE, \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)

#define SDLogCTrace()               LOG_C_MAYBE(SD_LOG_ASYNC_TRACE,   sdLogLevel, SD_LOG_FLAG_TRACE, \
SD_LOG_CONTEXT, @"%@[%p]: %@", THIS_FILE, self, __FUNCTION__)

#define SDLogCTrace2(frmt, ...)     LOG_C_MAYBE(SD_LOG_ASYNC_TRACE,   sdLogLevel, SD_LOG_FLAG_TRACE, \
SD_LOG_CONTEXT, frmt, ##__VA_ARGS__)
