#import <Foundation/Foundation.h>

@class SDService;

/**
 The names of the notifications used to notify about found/lost services and errors.
 */
extern NSString* SDServiceFound;
extern NSString* SDServiceLost;
extern NSString* SDServiceDiscoveryError;

/**
 The block definitions for when a service was found/lost or an error occured.
 */
typedef void (^SDServiceFoundHandler)(SDService* service, BOOL ownService);
typedef void (^SDServiceLostHandler)(SDService* service);
typedef void (^SDErrorHandler)(NSDictionary* errorInfo);

/**
 Provides methods to publish and search for services on the network.
 
 Instead of a delegate this class uses notifications to inform about found/lost
 services.
 */
@interface SDServiceDiscovery : NSObject<NSNetServiceBrowserDelegate, NSNetServiceDelegate>

/**
 Indicates whether at least one search is currently running.
 */
@property (nonatomic, readonly) BOOL isSearching;

/**
 Indicates whether at least one service is currently being published.
 */
@property (nonatomic, readonly) BOOL isPublishing;

/**
 Stops publishing and searching.
 */
- (void)stop;

/**
 Stops all searches for services.
 */
- (void)stopSearching;

/**
 Stops publishing all services.
 */
- (void)stopPublishing;

/**
 Search for services of the specfied type in the default domain.
 
 @param type Type of the services to search for.
 
 `type` must contain both the service type and transport layer information. To
 ensure that the mDNS responder searches for services, as opposed to hosts,
 prefix both the service name and transport layer name with an underscore
 character ("_"). For example, to search for an HTTP service on TCP, you would
 use the type string `@"_http._tcp."`. Note that the period character at the end
 of the string, which indicates that the domain name is an absolute name, is
 required. It will be automatically set if it does not exists.

 @return Returns NO if already searching for the type and domain.
 */
- (BOOL)searchForServicesOfType:(NSString*)type;

/**
 Search for services of the specified type in the specified domain.

 @param type Type of the services to search for.
 
 `type` must contain both the service type and transport layer information. To
 ensure that the mDNS responder searches for services, as opposed to hosts,
 prefix both the service name and transport layer name with an underscore
 character ("_"). For example, to search for an HTTP service on TCP, you would
 use the type string `@"_http._tcp."`. Note that the period character at the end
 of the string, which indicates that the domain name is an absolute name, is
 required. It will be automatically set if it does not exists.
 
 @param domain Domain name in which to perform the search.

 @return Returns NO if already searching for the type and domain.
 */
- (BOOL)searchForServicesOfType:(NSString*)type
                       inDomain:(NSString*)domain;

/**
 Stop searching for services of the specified type in the default domain.

 @param type Type of the search to stop.
 */
- (void)stopSearchingForServicesOfType:(NSString*)type;

/**
 Stop searching for services of the specified type in the specified domain.

 @param type Type of the search to stop.
 @param domain Domain name in which to stop the search.
 */
- (void)stopSearchingForServicesOfType:(NSString*)type
                              inDomain:(NSString*)domain;

/**
 Publish a service with the system defined name in the default domain.

 @param type Type of the service to publish.
 
 `type` must contain both the service type and transport layer information. To
 ensure that the mDNS responder searches for services, as opposed to hosts,
 prefix both the service name and transport layer name with an underscore
 character ("_"). For example, to publish an HTTP service on TCP, you would
 use the type string `@"_http._tcp."`. Note that the period character at the end
 of the string, which indicates that the domain name is an absolute name, is
 required. It will be automatically set if it does not exists.

 @param port Port of the service to publish.

 @return Returns NO if already publishing a service with the specified parameters.
 */
- (BOOL)publishServiceOfType:(NSString*)type
                      onPort:(int)port;

/**
 Publish a service in the default domain.
 
 The TXT record of the published service will be not set.

 @param type Type of the service to publish.
 
 `type` must contain both the service type and transport layer information. To
 ensure that the mDNS responder searches for services, as opposed to hosts,
 prefix both the service name and transport layer name with an underscore
 character ("_"). For example, to publish an HTTP service on TCP, you would
 use the type string `@"_http._tcp."`. Note that the period character at the end
 of the string, which indicates that the domain name is an absolute name, is
 required. It will be automatically set if it does not exists.
 
 @param port Port of the service to publish.
 @param name Name of the service to publish.

 @return Returns NO if already publishing a service with the specified parameters.
 */
- (BOOL)publishServiceOfType:(NSString*)type
                      onPort:(int)port
                    withName:(NSString*)name;

/**
 Publish a service.
 
 The TXT record of the published service will be not set.

 @param type Type of the service to publish.
 
 `type` must contain both the service type and transport layer information. To
 ensure that the mDNS responder searches for services, as opposed to hosts,
 prefix both the service name and transport layer name with an underscore
 character ("_"). For example, to publish an HTTP service on TCP, you would
 use the type string `@"_http._tcp."`. Note that the period character at the end
 of the string, which indicates that the domain name is an absolute name, is
 required. It will be automatically set if it does not exists.

 @param port Port of the service to publish.
 @param name Name of the service to publish.
 @param domain Domain of the service to publish.

 @return Returns NO if already publishing a service with the specified parameters.
 */
- (BOOL)publishServiceOfType:(NSString*)type
                      onPort:(int)port
                    withName:(NSString*)name
                    inDomain:(NSString*)domain;

/**
 Publish a service.

 @param type Type of the service to publish.
 
 `type` must contain both the service type and transport layer information. To
 ensure that the mDNS responder searches for services, as opposed to hosts,
 prefix both the service name and transport layer name with an underscore
 character ("_"). For example, to publish an HTTP service on TCP, you would
 use the type string `@"_http._tcp."`. Note that the period character at the end
 of the string, which indicates that the domain name is an absolute name, is
 required. It will be automatically set if it does not exists.

 @param port Port of the service to publish.
 @param name Name of the service to publish.
 @param domain Domain of the service to publish.
 @param TXTRecord TXT record of the service to publish.
 
 @return Returns NO if already publishing a service with the specified parameters.
 */
- (BOOL)publishServiceOfType:(NSString*)type
                      onPort:(int)port
                    withName:(NSString*)name
                    inDomain:(NSString*)domain
                   TXTRecord:(NSDictionary*)TXTRecord;

/**
 Stop publishing the service of the specified type on the specified port in the
 default domain.

 @param type Type of the service to stop.
 @param port Port of the service to stop.
 */
- (void)stopPublishingServiceOfType:(NSString*)type
                             onPort:(int)port;

/**
 Stop publishing the service of the specified type on the specified port in the
 specified domain.

 @param type Type of the service to stop.
 @param port Port of the service to stop.
 @param domain Domain of the service to stop.
 */
- (void)stopPublishingServiceOfType:(NSString*)type
                             onPort:(int)port
                           inDomain:(NSString*)domain;

/**
 Add a block to be executed when a service is found.
 
 To unregister the handler, you pass the object returned by this method to
 `removeHandler:`. You *must* invoke `removeHandler:` before any object
 specified is deallocated.

 @param handler The block to be executed when a service is found.
 
 @return Returns an opaque object to identify the handler.
 */
- (id)addHandlerForServiceFound:(SDServiceFoundHandler)handler;

/**
 Add a block to be executed when a service is lost.

 To unregister the handler, you pass the object returned by this method to
 `removeHandler:`. You *must* invoke `removeHandler:` before any object
 specified is deallocated.

 @param handler The block to be executed when a service is lost.
 
 @return Returns an opaque object to identify the handler.
 */
- (id)addHandlerForServiceLost:(SDServiceLostHandler)handler;

/**
 Add a block to be executed when an error occurs.

 To unregister the handler, you pass the object returned by this method to
 `removeHandler:`. You *must* invoke `removeHandler:` before any object
 specified is deallocated.

 @param handler The block to be executed when an error occurs.
 
 @return Returns an opaque object to identify the handler.
 */
- (id)addHandlerForServiceDiscoveryError:(SDErrorHandler)handler;

/**
 Remove a handler.
 
 Be sure to invoke `removeHandler:` before any object specified in a handler
 is deallocated.

 @param handler The handler to remove.
 */
- (void)removeHandler:(id)handler;

/**
 Set the log level of SDServiceDiscovery. For the defined log levels see
 `SDLogging.h`.
 
 @param logLevel Log level to set.
 */
- (void)setLogLevel:(int)logLevel;

@end
